@@ -12,6 +12,156 @@ Chat bots can join channels, listen to chat and reply to messages, commands, sub
     Please report all feature requests and bug requests to the `github page <https://github.com/Teekeks/pyTwitchAPI/issues>`_.
 
 
+********
+Commands
+********
+
+Chat commands are specific messages user can send in chat in order to trigger some action of your bot.
+
+Example:
+
+.. code-block::
+
+    <User123>: !say Hello world
+    <MyBot>: User123 asked me to say "Hello world"
+
+
+You can register listeners to chat commands using :const:`~twitchAPI.chat.Chat.register_command()`.
+
+The bot prefix can be set by using :const:`~twitchAPI.chat.Chat.set_prefix()`, the default is :code:`!`
+
+Your command listener function needs to be async and take in one parameter of type :const:`~twitchAPI.chat.ChatCommand`.
+
+Example:
+
+.. code-block:: python
+
+    async def say_command_handler(cmd: ChatCommand):
+        await cmd.reply(f'{cmd.user.name} asked me to say "{cmd.parameter}")
+
+    chat.register_command('say', say_command_handler)
+
+******
+Events
+******
+
+You can listen to different events happening in the chat rooms you joined.
+
+Generally you register a event listener using :const:`~twitchAPI.chat.Chat.register_event()`.
+The first parameter has to be of type :const:`~twitchAPI.types.ChatEvent` and the second one is your listener function.
+
+Those Listeners always have to be async functions taking in one parameter (the payload). The Payload type is described below.
+
+Example:
+
+.. code-block:: python
+
+    async def on_ready(cmd: EventData):
+        await cmd.chat.join_room('teekeks42')
+
+    chat.register_event(ChatEvent.READY, on_ready)
+
+Bot Ready
+=========
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.READY`
+- Payload: :const:`~twitchAPI.chat.EventData`
+
+This Event is triggered when the bot is stared up and ready to join channels.
+
+Message send
+============
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.MESSAGE`
+- Payload: :const:`~twitchAPI.chat.ChatMessage`
+
+This Event is triggered when someone wrote a message in a channel we joined
+
+Channel Subscription
+====================
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.SUB`
+- Payload: :const:`~twitchAPI.chat.ChatSub`
+
+This Event is triggered when someone subscribed to a channel we joined.
+
+Raid
+====
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.RAID`
+- Payload: :const:`dict`
+
+Triggered when a channel gets raided
+
+Channel config changed
+======================
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.ROOM_STATE_CHANGE`
+- Payload: :const:`~twitchAPI.chat.RoomStateChangeEvent`
+
+Triggered when a channel is changed (e.g. sub only mode was enabled)
+
+User channel join
+=================
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.JOIN`
+- Payload: :const:`~twitchAPI.chat.JoinEvent`
+
+Triggered when someone other than the bot joins a channel.
+
+.. note:: this will not always trigger, depending on channel size
+
+User channel leave
+==================
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.USER_LEFT`
+- Payload: :const:`~twitchAPI.chat.LeftEvent`
+
+.. note:: this will not always trigger, depending on channel size
+
+Bot channel join
+================
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.JOINED`
+- Payload: :const:`~twitchAPI.chat.JoinedEvent`
+
+Triggered when the bot joins a channel
+
+Bot channel leave
+=================
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.LEFT`
+- Payload: :const:`~twitchAPI.chat.LeftEvent`
+
+Triggered when the bot left a channel
+
+Message delete
+==============
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.MESSAGE_DELETE`
+- Payload: :const:`~twitchAPI.chat.MessageDeletedEvent`
+
+Triggered when a single message in a channel got deleted
+
+User messages cleared
+=====================
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.CHAT_CLEARED`
+- Payload: :const:`~twitchAPI.chat.ClearChatEvent`
+
+Triggered when a user was banned, timed out and/or all messaged from a user where deleted
+
+Bot recieves whisper message
+============================
+
+- ChatEvent: :const:`~twitchAPI.types.ChatEvent.WHISPER`
+- Payload: :const:`~twitchAPI.chat.WhisperEvent`
+
+Triggered when someone whispers to your bot.
+
+.. note:: You need the :const:`~twitchAPI.types.AuthScope.WHISPERS_READ` Auth Scope to receive this Event.
+
+
 ************
 Code example
 ************
@@ -99,26 +249,13 @@ Code example
     # lets run our setup
     asyncio.run(run())
 
-****************
-Available Events
-****************
-
-- :const:`~twitchAPI.types.ChatEvent.READY`: Triggered when the bot is stared up and ready to join channels
-- :const:`~twitchAPI.types.ChatEvent.MESSAGE`: Triggered when someone wrote a message in a channel we joined
-- :const:`~twitchAPI.types.ChatEvent.SUB`: Triggered when someone subscribed to a channel we joined
-- :const:`~twitchAPI.types.ChatEvent.RAID`: Triggered when a channel gets raided
-- :const:`~twitchAPI.types.ChatEvent.ROOM_STATE_CHANGE`: Triggered when a channel is changed (e.g. sub only mode was enabled)
-- :const:`~twitchAPI.types.ChatEvent.JOIN`: Triggered when someone other than the bot joins a channel. Note: this will not always trigger, depending on channel size
-- :const:`~twitchAPI.types.ChatEvent.JOINED`: Triggered when the bot joins a channel
-- :const:`~twitchAPI.types.ChatEvent.LEFT`: triggered when the bot left a channel
-- :const:`~twitchAPI.types.ChatEvent.MESSAGE_DELETE`: triggered when a message in a channel got deleted
-
 *******************
 Class Documentation
 *******************
 """
 import asyncio
 import dataclasses
+import datetime
 import sys
 import threading
 import traceback
@@ -131,22 +268,25 @@ from twitchAPI.object import TwitchUser
 from twitchAPI.helper import TWITCH_CHAT_URL, first
 from twitchAPI.types import ChatRoom, TwitchBackendException, AuthType, AuthScope, ChatEvent, MissingScopeException, UnauthorizedException
 
-from typing import List, Optional, Union, Callable, Dict
+from typing import List, Optional, Union, Callable, Dict, Awaitable, Any
 
-__all__ = ['ChatUser', 'EventData', 'ChatMessage', 'ChatCommand', 'ChatSub', 'Chat', 'ChatRoom', 'ChatEvent', 'RoomStateChangeEvent']
+__all__ = ['ChatUser', 'EventData', 'ChatMessage', 'ChatCommand', 'ChatSub', 'Chat', 'ChatRoom', 'ChatEvent', 'RoomStateChangeEvent',
+           'JoinEvent', 'JoinedEvent', 'LeftEvent', 'ClearChatEvent', 'WhisperEvent']
 
 
 class ChatUser:
     """Represents a user in a chat channel
     """
 
-    def __init__(self, chat, parsed):
+    def __init__(self, chat, parsed, name_override=None):
         self.chat: 'Chat' = chat
         """The :const:`twitchAPI.chat.Chat` instance"""
         self.name: str = parsed['source']['nick'] if parsed['source']['nick'] is not None else f'{chat.username}'
         """The name of the user"""
         if self.name[0] == ':':
             self.name = self.name[1:]
+        if name_override is not None:
+            self.name = name_override
         self.badge_info = parsed['tags'].get('badge-info')
         """All infos related to the badges of the user"""
         self.badges = parsed['tags'].get('badges')
@@ -170,9 +310,7 @@ class ChatUser:
 
 
 class EventData:
-    """Represents a basic chat event
-
-    :param ~twitchAPI.chat.Chat chat: represents the Chat Instance"""
+    """Represents a basic chat event"""
     def __init__(self, chat):
         self.chat: 'Chat' = chat
         """The :const:`twitchAPI.chat.Chat` instance"""
@@ -240,11 +378,13 @@ class JoinedEvent(EventData):
 
 
 class LeftEvent(EventData):
-    """When the bot left a room"""
-    def __init__(self, chat, channel_name, room):
+    """When the bot or a user left a room"""
+    def __init__(self, chat, channel_name, room, user):
         super(LeftEvent, self).__init__(chat)
         self.room_name: str = channel_name
         """the name of the channel the bot left"""
+        self.user_name: str = user
+        """The name of the user that left the chat"""
         self.cached_room: Optional[ChatRoom] = room
         """the cached room state, might bo Null"""
 
@@ -333,6 +473,48 @@ class ChatSub:
         return self.chat.room_cache.get(self._parsed['command']['channel'][1:])
 
 
+class ClearChatEvent(EventData):
+    
+    def __init__(self, chat, parsed):
+        super(ClearChatEvent, self).__init__(chat)
+        self.room_name: str = parsed['command']['channel'][1:]
+        """The name of the chat room the event happend in"""
+        self.room_id: str = parsed['tags'].get('room-id')
+        """The ID of the chat room the event happend in"""
+        self.user_name: str = parsed['parameters']
+        """The name of the user whos messages got cleared"""
+        self.duration: Optional[int] = int(parsed['tags']['ban-duration']) if parsed['tags']['ban-duration'] not in (None, '') else None
+        """duration of the timeout in seconds. None if user was not timed out"""
+        self.banned_user_id: Optional[str] = parsed['tags'].get('target-user-id')
+        """The ID of the user who got banned or timed out. if :const:`twitchAPI.chat.ClearChatEvent.duration` is None, the user was banned.
+        Will be None when the user was not banned nor timed out."""
+        self.sent_timestamp: int = int(parsed['tags'].get('tmi-sent-ts'))
+        """The timestamp the event happend at"""
+
+    @property
+    def room(self) -> Optional[ChatRoom]:
+        """The room this event was issued in. None on cache miss."""
+        return self.chat.room_cache.get(self.room_name)
+    
+
+class WhisperEvent(EventData):
+    
+    def __init__(self, chat, parsed):
+        super(WhisperEvent, self).__init__(chat)
+        self._parsed = parsed
+        self.message: str = parsed['parameters']
+        """The message that was send"""
+
+    @property
+    def user(self) -> ChatUser:
+        """The user that DMed your bot"""
+        return ChatUser(self.chat, self._parsed)
+
+
+COMMAND_CALLBACK_TYPE = Callable[[ChatCommand], Awaitable[None]]
+EVENT_CALLBACK_TYPE = Callable[[Any], Awaitable[None]]
+
+
 class Chat:
     """The chat bot instance"""
 
@@ -362,6 +544,8 @@ class Chat:
         self._room_join_locks = []
         self._room_leave_locks = []
         self._closing: bool = False
+        self.join_timeout: int = 10
+        """Time in seconds till a channel join attempt times out"""
 
     def __await__(self):
         t = asyncio.create_task(self._get_username())
@@ -505,7 +689,7 @@ class Chat:
         command_parts = raw_command_component.split(' ')
 
         if command_parts[0] in ('JOIN', 'PART', 'NOTICE', 'CLEARCHAT', 'HOSTTARGET', 'PRIVMSG',
-                                'USERSTATE', 'ROOMSTATE', '001', 'USERNOTICE', 'CLEARMSG'):
+                                'USERSTATE', 'ROOMSTATE', '001', 'USERNOTICE', 'CLEARMSG', 'WHISPER'):
             parsed_command = {
                 'command': command_parts[0],
                 'channel': command_parts[1]
@@ -591,7 +775,7 @@ class Chat:
 
     async def __connect(self, is_startup=False):
         self.logger.debug('connecting...')
-        if self.__connection is not None and self.__connection.open:
+        if self.__connection is not None and not self.__connection.closed:
             await self.__connection.close()
         retry = 0
         need_retry = True
@@ -638,6 +822,20 @@ class Chat:
 
     async def __task_receive(self):
         try:
+            handlers: Dict[str, Callable] = {
+                'PING': self._handle_ping,
+                'PRIVMSG': self._handle_msg,
+                '001': self._handle_ready,
+                'ROOMSTATE': self._handle_room_state,
+                'JOIN': self._handle_join,
+                'USERNOTICE': self._handle_user_notice,
+                'CLEARMSG': self._handle_clear_msg,
+                'CAP': self._handle_cap_reply,
+                'PART': self._handle_part,
+                'NOTICE': self._handle_notice,
+                'CLEARCHAT': self._handle_clear_chat,
+                'WHISPER': self._handle_whisper
+            }
             while not self.__connection.closed:
                 message = await self.__connection.receive()
                 if message.type == aiohttp.WSMsgType.TEXT:
@@ -645,21 +843,11 @@ class Chat:
                     for m in messages:
                         if len(m) == 0:
                             continue
+                        self.logger.debug(f'< {m}')
                         parsed = self._parse_irc_message(m)
                         # a message we don't know or don't care about
                         if parsed is None:
                             continue
-                        handlers: Dict[str, Callable] = {
-                            'PING': self._handle_ping,
-                            'PRIVMSG': self._handle_msg,
-                            '001': self._handle_ready,
-                            'ROOMSTATE': self._handle_room_state,
-                            'JOIN': self._handle_join,
-                            'USERNOTICE': self._handle_user_notice,
-                            'CLEARMSG': self._handle_clear_msg,
-                            'CAP': self._handle_cap_reply,
-                            'PART': self._handle_part
-                        }
                         handler = handlers.get(parsed['command']['command'])
                         if handler is not None:
                             asyncio.ensure_future(handler(parsed))
@@ -674,6 +862,22 @@ class Chat:
             # print('we are closing down!')
             return
 
+    async def _handle_whisper(self, parsed: dict):
+        e = WhisperEvent(self, parsed)
+        for handler in self._event_handler.get(ChatEvent.WHISPER, []):
+            t = asyncio.ensure_future(handler(e))
+            t.add_done_callback(self._task_callback)
+
+    async def _handle_clear_chat(self, parsed: dict):
+        e = ClearChatEvent(self, parsed)
+        for handler in self._event_handler.get(ChatEvent.CHAT_CLEARED, []):
+            t = asyncio.ensure_future(handler(e))
+            t.add_done_callback(self._task_callback)
+
+    async def _handle_notice(self, parsed: dict):
+        self.logger.debug(f'got NOTICE for channel {parsed["command"]["channel"]}: {parsed["tags"]["msg-id"]}')
+        pass
+
     async def _handle_clear_msg(self, parsed: dict):
         ev = MessageDeletedEvent(self, parsed)
         for handler in self._event_handler.get(ChatEvent.MESSAGE_DELETE, []):
@@ -704,13 +908,21 @@ class Chat:
 
     async def _handle_part(self, parsed: dict):
         ch = parsed['command']['channel'][1:]
-        if ch in self._room_leave_locks:
-            self._room_leave_locks.remove(ch)
-        room = self.room_cache.pop(ch, None)
-        e = LeftEvent(self, ch, room)
-        for handler in self._event_handler.get(ChatEvent.LEFT, []):
-            t = asyncio.ensure_future(handler(e))
-            t.add_done_callback(self._task_callback)
+        usr = parsed['source']['nick'][1:]
+        if usr == self.username:
+            if ch in self._room_leave_locks:
+                self._room_leave_locks.remove(ch)
+            room = self.room_cache.pop(ch, None)
+            e = LeftEvent(self, ch, room, usr)
+            for handler in self._event_handler.get(ChatEvent.LEFT, []):
+                t = asyncio.ensure_future(handler(e))
+                t.add_done_callback(self._task_callback)
+        else:
+            room = self.room_cache.get(ch)
+            e = LeftEvent(self, ch, room, usr)
+            for handler in self._event_handler.get(ChatEvent.USER_LEFT, []):
+                t = asyncio.ensure_future(handler(e))
+                t.add_done_callback(self._task_callback)
 
     async def _handle_user_notice(self, parsed: dict):
         if parsed['tags'].get('msg-id') == 'raid':
@@ -781,32 +993,63 @@ class Chat:
     # user functions
     ##################################################################################################################################################
 
-    def register_command(self, name: str, handler: Callable) -> bool:
+    def register_command(self, name: str, handler: COMMAND_CALLBACK_TYPE) -> bool:
         """Register a command
 
         :param name: the name of the command
-        :param handler: The event handler"""
+        :param handler: The event handler
+        :raises ValueError: if handler is not a coroutine"""
+        if not asyncio.iscoroutinefunction(handler):
+            raise ValueError('handler needs to be a async function which takes one parameter')
         name = name.lower()
         if self._command_handler.get(name) is not None:
             return False
         self._command_handler[name] = handler
         return True
 
-    def register_event(self, event: ChatEvent, handler: Callable):
+    def unregister_command(self, name: str) -> bool:
+        """Unregister a already registered command.
+
+        :param name: the name of the command to unregister
+        :return: True if the command was unregistered, otherwise false
+        """
+        name = name.lower()
+        if self._command_handler.get(name) is None:
+            return False
+        self._command_handler.pop(name, None)
+        return True
+
+    def register_event(self, event: ChatEvent, handler: EVENT_CALLBACK_TYPE):
         """Register a event handler
 
         :param event: The Event you want to register the handler to
-        :param handler: The handler you want to register."""
+        :param handler: The handler you want to register.
+        :raises ValueError: if handler is not a coroutine"""
+        if not asyncio.iscoroutinefunction(handler):
+            raise ValueError('handler needs to be a async function which takes one parameter')
         if self._event_handler.get(event) is None:
             self._event_handler[event] = [handler]
         else:
             self._event_handler[event].append(handler)
 
+    def unregister_event(self, event: ChatEvent, handler: EVENT_CALLBACK_TYPE) -> bool:
+        """Unregister a handler from a event
+
+        :param event: The Event you want to unregister your handler from
+        :param handler: The handler you want to unregister
+        :return: Returns true when the handler was removed from the event, otherwise false
+        """
+        if self._event_handler.get(event) is None or handler not in self._event_handler.get(event):
+            return False
+        self._event_handler[event].remove(handler)
+        return True
+
     async def join_room(self, chat_rooms: Union[List[str], str]):
         """ join one or more chat rooms\n
-        Will only exit once all given chat rooms where successfully joined
+        Will only exit once all given chat rooms where successfully joined or :const:`twitchAPI.chat.Chat.join_timeout` run out.
 
         :param chat_rooms: the Room or rooms you want to leave
+        :returns: list of channels that could not be joined
         """
         if isinstance(chat_rooms, str):
             chat_rooms = [chat_rooms]
@@ -816,8 +1059,13 @@ class Chat:
             self._room_join_locks.append(r)
         await self._send_message(f'JOIN {room_str}')
         # wait for us to join all rooms
-        while any([r in self._room_join_locks for r in target]):
+        timeout = datetime.datetime.now() + datetime.timedelta(seconds=self.join_timeout)
+        while any([r in self._room_join_locks for r in target]) and timeout > datetime.datetime.now():
             await asyncio.sleep(0.01)
+        failed_to_join = [r for r in self._room_join_locks if r in target]
+        for r in failed_to_join:
+            self._room_join_locks.remove(r)
+        return failed_to_join
 
     async def send_raw_irc_message(self, message: str):
         """Send a raw IRC message
