@@ -406,6 +406,7 @@ class Stream(TwitchObject):
     thumbnail_url: str
     tag_ids: List[str]
     is_mature: bool
+    tags: List[str]
 
 
 class StreamMarker(TwitchObject):
@@ -540,6 +541,7 @@ class ChannelInformation(TwitchObject):
     broadcaster_language: str
     title: str
     delay: int
+    tags: List[str]
 
 
 class SearchChannelResult(ChannelInformation):
@@ -885,3 +887,11 @@ class Chatter(TwitchObject):
 class GetChattersResponse(AsyncIterTwitchObject[Chatter]):
     data: List[Chatter]
     total: int
+
+
+class ShieldModeStatus(TwitchObject):
+    is_active: bool
+    moderator_id: str
+    moderator_login: str
+    moderator_name: str
+    last_activated_at: Optional[datetime]
