@@ -76,6 +76,8 @@ class AuthScope(Enum):
     CHANNEL_MANAGE_VIPS = 'channel:manage:vips'
     USER_MANAGE_WHISPERS = 'user:manage:whispers'
     MODERATOR_READ_CHATTERS = 'moderator:read:chatters'
+    MODERATOR_READ_SHIELD_MODE = 'moderator:read:shield_mode'
+    MODERATOR_MANAGE_SHIELD_MODE = 'moderator:manage:shield_mode'
 
 
 class ModerationEventType(Enum):
@@ -246,6 +248,7 @@ class SoundtrackSourceType(Enum):
 # CHAT
 
 class ChatEvent(Enum):
+    """Represents the possible events to listen for using :const:`~twitchAPI.chat.Chat.register_event()`"""
     READY = 'ready'
     """Triggered when the bot is started up and ready"""
     MESSAGE = 'message'
@@ -262,8 +265,14 @@ class ChatEvent(Enum):
     """Triggered when the bot joins a chat channel"""
     LEFT = 'left'
     """Triggered when the bot leaves a chat channel"""
+    USER_LEFT = 'user_left'
+    """Triggered when a user leaves a chat channel"""
     MESSAGE_DELETE = 'message_delete'
     """Triggered when a message gets deleted from a channel"""
+    CHAT_CLEARED = 'chat_cleared'
+    """Triggered when a user was banned, timed out or all messaged from a user where deleted"""
+    WHISPER = 'whisper'
+    """Triggered when someone whispers to your bot. NOTE: You need the :const:`~twitchAPI.types.AuthScope.WHISPERS_READ` Auth Scope to get this Event."""
 
 
 @dataclass
